#ifndef DUALMODE_H_INCLUDED
#define DUALMODE_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

#define INCL_WIN
#include <os2.h>

// Use the following definition to enable a debug log to be created for
// initialization of the client.  Note that this changes the size of the
// MGLSC_state structure below and will require an application to be
// recompiled to avoid structure alignment errors.
#define MGLSC_DEBUG_LOG

#include "dive.h"

#ifdef MGLSC_DEBUG_LOG
  #include <stdio.h>
#endif

struct vidmode_descrip
{
  int width, height;
  char depth;
};

struct MGLSC_state
{
  // DATA FOR INTERNAL USE ONLY
  // ==========================
  ULONG listener_thread, server_pid, pm_thread;
    // Thread IDs and process IDs
  ULONG MGL_session;
    // Session ID for full screen session (for use with DosSelectSession)
  void *shared_packet;
    // Packet used for communication with MGLServer
  HQUEUE command_queue, listener_queue;
    // Queues for communicating with MGLServer
  HEV client_wakeup;
    // Wake-up call for synchronous commands to MGLServer
  HDIVE diveinst;
    // DIVE instance
  ULONG divebufnum, depthfourcc;
    // DIVE buffer handle and the fourcc representation of current color depth
  SETUP_BLITTER BlSet;
    // Parameters for PM DIVE operation
  char palette_dirty;
    // Internal flag marking the palette as dirty
    // Shadow palette is contained in the shared packet
  int fswidth, fsheight;
    // Dimensions of the full screen video mode chosen
  char *winclassname, *wintitle, windowdecor;
  struct vidmode_descrip *allowedmodes;
    // Stored internally, but developer passes these in
  char vidmode_reinit;
    // Flag that full screen session needs to change its video mode
    // (PM mode can change immediately, but full screen session needs
    //  to wait until it's reactivated to change.)
  char disallow_blit;
    // Temporary disallow blitting when screen mode is being changed.
  #ifdef MGLSC_DEBUG_LOG
    FILE *dbfp;
    // Debug log file handle
  #endif


  // DATA THAT COULD BE USEFUL FOR DEVELOPERS
  // ========================================
  char autoBlitVidBufferOnRefresh;
    // Toggles whether or not the video buffer should automatically be
    // blitted when the screen mode changes.
  char isFullScreen;
    // 1 if full screen session is active
  void *vidbuffer;
    // Video buffer.  Write your image HERE.
  HEV framesync_sem;
    // Posted when a blit has been completed
  int width, height, depth;
    // Dimensions of the video buffer
  HWND clientwin, framewin;
    // Client and frame window handles
  char stretchblit;
    // Enable an image to be stretched as it is blitted in full screen mode
  char custommodes;
    // Whether or not the use of custom on-the-fly video modes is allowed
};

typedef enum
{
  MGLSC_KEYBOARD_MAKE, MGLSC_KEYBOARD_BREAK, MGLSC_MOUSE_MOVE,
  MGLSC_MOUSE_BUTTON_PRESS, MGLSC_MOUSE_BUTTON_RELEASE,
  MGLSC_JOYSTICK_MOVE, MGLSC_JOYSTICK_BUTTON,
  MGLSC_JOYSTICK2_MOVE, MGLSC_JOYSTICK2_BUTTON
} InputEventType;

#define MGLSC_WINDOW_DECOR_MENU     1
#define MGLSC_WINDOW_DECOR_ICON     2
#define MGLSC_WINDOW_DECOR_TASKLIST 4
#define MGLSC_WINDOW_DECOR_MINMAX   8
#define MGLSC_WINDOW_DECOR_SYSMENU  16
#define MGLSC_WINDOW_DECOR_TITLEBAR 32

#define MGLSC_WINDOW_DECOR_PLAIN    0
#define MGLSC_WINDOW_DECOR_ALL      63

struct MGLSC_event_handlers
{
  MRESULT EXPENTRY (*AdditionalWindowProc) ( HWND, ULONG, MPARAM, MPARAM );
  // If this is set, it is called from the window procedure after normal window
  // processing is complete.

  void (*WindowInitializer) ( void );
  // This is called just after the window is created.  You can use this
  // time to enable or disable any menu items and set the window position
  // before it pops up.

  void (*ProcessInput) ( InputEventType, short, short );
  // In the keyboard messages, the first data parameter is the keyboard
  // scancode, and the second is unused.

  // In mouse and joystick movements the 2 data parameters are X and Y
  // position respectively.

  // In mouse and joystick button press messages, the first data parameter
  // is a bitmask of the button(s) pressed, and the second is unused.

  // Note that this function can be called by either the PM thread or the
  // listener queue thread (handling MGL input).  Don't rely on calling
  // any Win* API functions here.

  void (*WindowClosed) ( void );
  // Called on a WM_CLOSE message.  Use this opportunity to tell your core
  // to close up shop.  (Don't worry about closing the MGL session.  This
  // should be handled automatically.)

  void (*PMMenuAction) ( ULONG );
  // Handle any PM menu interaction here.  Command ID is passed in.
};

struct PMkey {
	char *description;
	unsigned char MGL_key;
};

#define DEFINED_KEYS 111

//
// API CALLS
// =========
//

void MGLSC_init( int width, int height, int depth,
 struct MGLSC_event_handlers *eventhandler, char *winclassname,
 char *wintitle, struct vidmode_descrip *allowedmodes,
 char windowDecorations, char allowCustomModes,
 char useJoystickIfAvailable );
// eventhandler is copied to the global struct.
// The mglclient global struct is cleared at the beginning of this call.
// You can customize the window class name and window titlebar text here.
// Note that the classname and window title strings should be
// permanently allocated somewhere before calling, not allocated on a
// stack frame that could be popped (use a global character array).
// If you wish to enable joystick input, set useJoystickIfAvailable to 1
// and your ProcessInput function will receive data from the joystick
// every time MGLSC_flushUserInput (see below) is called.

void MGLSC_reinit( int width, int height, int depth,
 char allowCustomModes, char useJoystickIfAvailable );
// Change any of the above parameters after MGLSC_init has already been
// called without shutting everything down and restarting.

struct MGLSC_state *MGLSC_clientState( void );
// Returns a pointer to much useful information about the state of this
// client.  You should avoid modifying any parameters in this structure
// unless you *really* know what you're doing.  Use it for information
// only.

void MGLSC_flushUserInput( void );
// Makes sure that user input is acknowledged and passed to the
// appropriate event handler routine.  This call returns immediately
// and does not wait for any user input to occur.  This should be called
// about once per frame.

const char * MGLSC_getKeyName( char PMkeycode );
// Returns a string describing the key passed in.  This is only really
// useful for GUI configuration dialogs, so it accepts key codes 
// received from WM_CHAR KC_SCANCODE messages.  If you want the name
// associated with an MGL key code, you need to call the translate function
// below.

char MGLSC_MGLkeyToPMKeycode( char MGLkeycode );
// Translates key codes received through ProcessInput or the MGL evt_
// API calls to equivalent PM scancodes.  Note that this involves a
// reverse-lookup and can be slow.  Don't use this too often if at all.

char MGLSC_PMkeyToMGLKeycode( char PMkeycode );
// Translates key codes received through a WM_CHAR KC_SCANCODE message
// to the equivalent MGL keycode.  This is not needed if the key code
// is obtained in your ProcessInput event.  These values are translated
// automatically.  This is only needed if you've got your own PM window
// getting keyboard input.

char MGLSC_isJoystickPresent( void );
// Returns whether or not a supported joystick driver was detected.

char MGLSC_getNumJoysticks( void );
// Returns the number of joysticks present.

char MGLSC_getNumJoyButtons( LONG whichStick );
// Returns the number of buttons on the joystick selected (0-based).

void MGLSC_mouseGrab( char yesNo );
// When yesNo is non-zero, this causes the application to grab control of
// the mouse pointer, make the system mouse cursor invisible, and accept
// and log mouse movement and button information.  Mouse grabbing is
// automatically activated when in full screen mode, so this setting only
// applies to PM operation.  You will not receive mouse input in PM mode
// unless mouseGrab is activated.  Mouse grabbing is automatically
// deactivated if the window focus changes in PM mode.  On startup, this
// library defaults to having grabbing deactivated.

void MGLSC_setMouseModeRelative( char isRelative );
// Toggles whether mouse movement should be given as relative or absolute
// measures of mouse position.  When relative mode is set on, only the
// change in the mouse position since the last event flush will be
// returned.  When absolute mode is specified (by passing a 0 to this
// function), an absolute position is specified in terms of the size of
// the playfield.  On startup, this library defaults to relative mode.
// This function can also be used to reset the absolute position back to
// the center of the playfield if absolute mode is selected.


// HERE'S HOW YOU COMMUNICATE WITH THIS INTERFACE...
// =================================================
// Simply call MGLSC_init with the appropriate parameters.
// You can blit by posting a WM_BLITFRAME to the client window handle.
// This will blit the entire frame and update the palette if needed.
// The semaphore "framesync_sem" will be posted when the blit is complete.
// If you wish to update the palette, cast a pointer to the shared packet
//  to be a MGL_SERVER_COLORS_SET_PACKET *.  Next, access the "colors"
//  member of this structure and modify it as you like.  Finally, set the
//  MGLSC_clientState()->palette_dirty to 1.
// When the user specifies that they wish to go full screen or come back,
//  simply post a WM_TOGGLEFS to the client window handle.  If this is the
//  first time you've gone full screen, MGLServer will be initialized without
//  special intervention required by the developer.
// To receive user input, periodically call MGLSC_flushUserInput.
//  IMPORTANT NOTE: you MUST call this function to have the system acknowledge
//  CTRL-ESC to switch tasks, otherwise, the user can be held captive by your
//  application.  You should probably call this function with every frame
//  blit.  This function will also ensure that mouse and joystick input is
//  acknowledged.
// To capture the video buffer of your application to the clipboard, simply
//  post a WM_CLIPCAPTURE to MGLSC_clientState()->clientwin.  You will hear
//  a WA_WARNING if it succeeded or a WA_ERROR if it failed to capture.
// The WM_MOUSE_RECENTER message is used internally when mouse grabbing is
//  active.
// To adjust the window size and position to convenient values, use the
//  WM_SETWINSIZE message.  MPARAM1 is the scaling factor and MPARAM2 is
//  a flag specifying whether or not you want the window centered on the
//  desktop.
// To shut down both the PM and MGL sessions cleanly, issue a WM_CLOSE
//  message to the client window handle and do a DosWaitThread on the
//  MGLSC_clientState()->pm_thread.
//
// **IMPORTANT NOTE**
// Some audio drivers cause a problem with MGL when they are playing sound
// while MGL is initializing.  To avoid this problem, the first time the
// user requests to go full screen, you should shut down your application's
// audio interface and re-enable it when the mode switch is complete.  You
// can time this correctly simply by using WinSendMsg to pass the WM_TOGGLEFS
// message rather than WinPostMsg.  When WinSendMsg returns, it is safe to
// re-enable audio in your application.

#define WM_BLITFRAME      WM_USER
#define WM_TOGGLEFS       WM_USER+1
#define WM_CLIPCAPTURE    WM_USER+2
#define WM_MOUSE_RECENTER WM_USER+3
#define WM_SETWINSIZE     WM_USER+4


// This interface reserves WM_USER and WM_USER+1, so the first real user
//  message starts at WM_USER+2 or WM_USER_MSG as I've defined below.
#define WM_USER_MSG      WM_USER+50

// Some folks might not have these definitions
#ifndef WM_VRNENABLE
#define WM_VRNENABLE 0x7f
#endif
#ifndef WM_VRNDISABLE
#define WM_VRNDISABLE 0x7e
#endif

#ifdef __cplusplus
}
#endif

#endif

